#include <stdio.h>
#include <stdint.h>
#include <locale.h>
#include <cstdlib>

void print_in_hex(uint8_t byte)
{
    printf("%04X ", byte);
}

void print_in_hex(const void* data, size_t size)
{
    const uint8_t* bytes = (const uint8_t*)data;
    for (size_t i = 0; i < size; i++)
    {
        print_in_hex(bytes[i]);
        if ((i + 1) % 16 == 0)
            printf("\n");
    }
    printf("\n");
}

void print_in_binary(uint8_t byte)
{
    for (int i = 7; i >= 0; i--)
    {
        printf("%d", (byte >> i) & 1);
        if (i % 4 == 0)
            printf(" ");
    }
}

void print_in_binary(const void* data, size_t size)
{
    const uint8_t* bytes = (const uint8_t*)data;
    for (size_t i = 0; i < size; ++i)
    {
        print_in_binary(bytes[i]);
        if ((i + 1) % 4 == 0)
            printf("\n");
    }
    printf("\n");
}

int main()
{
    uint8_t byte = 127;

    printf("Source number: %hhu\n", byte);
    printf("Number in hexadecimal: ");
    print_in_hex(byte);
    printf("\n");
    printf("Binary number: ");
    print_in_binary(byte);
    printf("\n");
    printf("\n");
    const int SIZE = 10;
    uint8_t data[] = { 0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0xF0, 0x00, 0xA1 };
    size_t size = sizeof(data);

    printf("Source data block in decimal form\n");
    for (int i = 0; i < SIZE; ++i)
        printf("%hhu ", data[i]);
    printf("\n");

    printf("Data block in hexadecimal\n");
    print_in_hex(data, size);
    printf("Binary data block\n");
    print_in_binary(data, size);

    system("pause");
    return 0;
}
